@@ -38,7 +38,8 @@ LOG = logging.getLogger(__name__)
 # LOG.setLevel(logging.DEBUG)
 # logging.basicConfig(level=logging.DEBUG)
 
-OVN_TABLES = ["Port_Binding", "Chassis", "Datapath_Binding", "Load_Balancer"]
+OVN_TABLES = ["Port_Binding", "Chassis", "Datapath_Binding", "Load_Balancer",
+              "Chassis_Private", "Logical_DP_Group"]
 
 
 class OVNBGPDriver(driver_api.AgentDriverBase):
@@ -92,56 +93,34 @@ class OVNBGPDriver(driver_api.AgentDriverBase):
             LOG.info("Configured allowed address scopes: %s",
                      ", ".join(self.allowed_address_scopes))
 
-        events = ()
-        for event in self._get_events():
-            event_class = getattr(watcher, event)
-            events += (event_class(self),)
-
         self._post_fork_event.clear()
-        # TODO(lucasagomes): The OVN package in the ubuntu LTS is old
-        # and does not support Chassis_Private. Once the package is updated
-        # we can remove this fallback mode.
-        try:
-            try:
-                self.sb_idl = ovn.OvnSbIdl(
-                    self.ovn_remote,
-                    chassis=self.chassis,
-                    tables=OVN_TABLES + ["Chassis_Private",
-                                         "Logical_DP_Group"],
-                    events=events).start()
-            except AssertionError:
-                self.sb_idl = ovn.OvnSbIdl(
-                    self.ovn_remote,
-                    chassis=self.chassis,
-                    tables=OVN_TABLES + ["Chassis_Private"],
-                    events=events).start()
-        except AssertionError:
-            events.remove("ChassisPrivateCreateEvent")
-            self.sb_idl = ovn.OvnSbIdl(
-                self.ovn_remote,
-                chassis=self.chassis,
-                tables=OVN_TABLES,
-                events=events).start()
+
+        events = self._get_events()
+        self.sb_idl = ovn.OvnSbIdl(
+            self.ovn_remote,
+            chassis=self.chassis,
+            tables=OVN_TABLES,
+            events=events).start()
 
         # Now IDL connections can be safely used
         self._post_fork_event.set()
 
     def _get_events(self):
-        events = set(["PortBindingChassisCreatedEvent",
-                      "PortBindingChassisDeletedEvent",
-                      "FIPSetEvent",
-                      "FIPUnsetEvent",
-                      "OVNLBMemberCreateEvent",
-                      "OVNLBMemberDeleteEvent",
-                      "ChassisCreateEvent",
-                      "ChassisPrivateCreateEvent",
-                      "LocalnetCreateDeleteEvent"])
+        events = {watcher.PortBindingChassisCreatedEvent(self),
+                  watcher.PortBindingChassisDeletedEvent(self),
+                  watcher.FIPSetEvent(self),
+                  watcher.FIPUnsetEvent(self),
+                  watcher.OVNLBMemberCreateEvent(self),
+                  watcher.OVNLBMemberDeleteEvent(self),
+                  watcher.ChassisCreateEvent(self),
+                  watcher.ChassisPrivateCreateEvent(self),
+                  watcher.LocalnetCreateDeleteEvent(self)}
         if self._expose_tenant_networks:
-            events.update(["SubnetRouterAttachedEvent",
-                           "SubnetRouterDetachedEvent",
-                           "TenantPortCreatedEvent",
-                           "TenantPortDeletedEvent",
-                           "OVNLBVIPPortEvent"])
+            events.update({watcher.SubnetRouterAttachedEvent(self),
+                           watcher.SubnetRouterDetachedEvent(self),
+                           watcher.TenantPortCreatedEvent(self),
+                           watcher.TenantPortDeletedEvent(self),
+                           watcher.OVNLBVIPPortEvent(self)})
         return events
 
     @lockutils.synchronized('bgp')
@@ -325,6 +304,8 @@ class OVNBGPDriver(driver_api.AgentDriverBase):
     def _expose_provider_port(self, port_ips, provider_datapath,
                               bridge_device=None, bridge_vlan=None,
                               lladdr=None, proxy_cidrs=None):
+        if proxy_cidrs is None:
+            proxy_cidrs = []
         if not bridge_device and not bridge_vlan:
             bridge_device, bridge_vlan = self._get_bridge_for_datapath(
                 provider_datapath)
@@ -354,7 +335,7 @@ class OVNBGPDriver(driver_api.AgentDriverBase):
             if wire_utils.wire_provider_port(
                     self.ovn_routing_tables_routes, self.ovs_flows, port_ips,
                     bridge_device, bridge_vlan, localnet,
-                    self.ovn_routing_tables, proxy_cidrs, lladdr):
+                    self.ovn_routing_tables, proxy_cidrs, lladdr=lladdr):
                 # Expose the IP now that it is connected
                 bgp_utils.announce_ips(port_ips)
                 return True
@@ -369,7 +350,7 @@ class OVNBGPDriver(driver_api.AgentDriverBase):
         # specific case for ovn-lb vips on tenant networks
         if not port.mac and not port.chassis and not port.up[0]:
             ext_n_cidr = port.external_ids.get(
-                constants.OVN_CIDRS_EXT_ID_KEY)
+                constants.OVN_CIDRS_EXT_ID_KEY, "")
             if ext_n_cidr:
                 ovn_lb_ip = ext_n_cidr.split(" ")[0].split("/")[0]
                 bgp_utils.announce_ips([ovn_lb_ip])
@@ -390,7 +371,8 @@ class OVNBGPDriver(driver_api.AgentDriverBase):
             if port.mac == ['unknown']:
                 # Handling the case for unknown MACs when configdrive is used
                 # instead of dhcp
-                n_cidrs = port.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY)
+                n_cidrs = port.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY,
+                                                "")
                 port_ips = [ip.split("/")[0] for ip in n_cidrs.split(" ")]
             else:
                 port_ips = port.mac[0].strip().split(' ')[1:]
@@ -415,6 +397,8 @@ class OVNBGPDriver(driver_api.AgentDriverBase):
     def _withdraw_provider_port(self, port_ips, provider_datapath,
                                 bridge_device=None, bridge_vlan=None,
                                 lladdr=None, proxy_cidrs=None):
+        if proxy_cidrs is None:
+            proxy_cidrs = []
         # Withdraw IP before disconnecting it
         bgp_utils.withdraw_ips(port_ips)
 
@@ -428,7 +412,8 @@ class OVNBGPDriver(driver_api.AgentDriverBase):
         try:
             return wire_utils.unwire_provider_port(
                 self.ovn_routing_tables_routes, port_ips, bridge_device,
-                bridge_vlan, self.ovn_routing_tables, proxy_cidrs, lladdr)
+                bridge_vlan, self.ovn_routing_tables, proxy_cidrs,
+                lladdr=lladdr)
         except Exception as e:
             LOG.exception("Unexpected exception while unwiring provider port: "
                           "%s", e)
@@ -580,10 +565,10 @@ class OVNBGPDriver(driver_api.AgentDriverBase):
                     # NOTE: This is neutron specific as we need the provider
                     # prefix to add the ndp proxy
                     n_cidr = row.external_ids.get(
-                        constants.OVN_CIDRS_EXT_ID_KEY)
+                        constants.OVN_CIDRS_EXT_ID_KEY, "").split()
                     exposed_port = self._expose_provider_port(
                         ips, row.datapath, bridge_device, bridge_vlan, None,
-                        [n_cidr])
+                        n_cidr)
                 else:
                     exposed_port = self._expose_provider_port(ips,
                                                               row.datapath)
@@ -720,11 +705,11 @@ class OVNBGPDriver(driver_api.AgentDriverBase):
                             # NOTE: This is neutron specific as we need the
                             # provider prefix to add the ndp proxy
                             n_cidr = row.external_ids.get(
-                                constants.OVN_CIDRS_EXT_ID_KEY)
+                                constants.OVN_CIDRS_EXT_ID_KEY, "").split()
                 if n_cidr:
                     withdrawn_port = self._withdraw_provider_port(
                         ips, row.datapath, bridge_device, bridge_vlan, None,
-                        [n_cidr])
+                        n_cidr)
                 else:
                     withdrawn_port = self._withdraw_provider_port(ips,
                                                                   row.datapath)
@@ -759,6 +744,7 @@ class OVNBGPDriver(driver_api.AgentDriverBase):
             if (associated_port and (
                     self.sb_idl.is_port_on_chassis(
                         associated_port, self.chassis) or
+                    self.sb_idl.is_port_without_chassis(associated_port) or
                     self.sb_idl.is_port_deleted(associated_port))):
                 if not self.sb_idl.is_provider_network(row.datapath):
                     # Only exposing IPs if the associated network is a
