@@ -27,6 +27,7 @@ from ovn_bgp_agent.drivers.openstack.utils import ovn
 from ovn_bgp_agent.drivers.openstack.utils import ovs
 from ovn_bgp_agent.drivers.openstack.watchers import evpn_watcher as \
     watcher
+from ovn_bgp_agent.utils import helpers
 from ovn_bgp_agent.utils import linux_net
 
 
@@ -35,7 +36,7 @@ LOG = logging.getLogger(__name__)
 # LOG.setLevel(logging.DEBUG)
 # logging.basicConfig(level=logging.DEBUG)
 
-OVN_TABLES = ["Port_Binding", "Chassis", "Datapath_Binding"]
+OVN_TABLES = ["Port_Binding", "Chassis", "Datapath_Binding", "Chassis_Private"]
 EVPN_INFO = collections.namedtuple(
     'EVPNInfo', ['vrf_name', 'lo_name', 'bridge_name', 'vxlan_name',
                  'veth_vrf', 'veth_ovs', 'vlan_name'])
@@ -71,43 +72,28 @@ class OVNEVPNDriver(driver_api.AgentDriverBase):
         self.ovn_remote = self.ovs_idl.get_ovn_remote()
         LOG.debug("Loaded chassis %s.", self.chassis)
 
-        events = ()
-        for event in self._get_events():
-            event_class = getattr(watcher, event)
-            events += (event_class(self),)
-
         self._post_fork_event.clear()
-        # TODO(lucasagomes): The OVN package in the ubuntu LTS is old
-        # and does not support Chassis_Private. Once the package is updated
-        # we can remove this fallback mode.
-        try:
-            self.sb_idl = ovn.OvnSbIdl(
-                self.ovn_remote,
-                chassis=self.chassis,
-                tables=OVN_TABLES + ["Chassis_Private"],
-                events=events).start()
-        except AssertionError:
-            events.remove("ChassisPrivateCreateEvent")
-            self.sb_idl = ovn.OvnSbIdl(
-                self.ovn_remote,
-                chassis=self.chassis,
-                tables=OVN_TABLES,
-                events=events).start()
+
+        events = self._get_events()
+        self.sb_idl = ovn.OvnSbIdl(
+            self.ovn_remote,
+            chassis=self.chassis,
+            tables=OVN_TABLES,
+            events=events).start()
 
         # Now IDL connections can be safely used
         self._post_fork_event.set()
 
     def _get_events(self):
-        events = set(["PortBindingChassisCreatedEvent",
-                      "PortBindingChassisDeletedEvent",
-                      "SubnetRouterAttachedEvent",
-                      "SubnetRouterDetachedEvent",
-                      "TenantPortCreatedEvent",
-                      "TenantPortDeletedEvent",
-                      "ChassisCreateEvent",
-                      "ChassisPrivateCreateEvent",
-                      "LocalnetCreateDeleteEvent"])
-        return events
+        return {watcher.PortBindingChassisCreatedEvent(self),
+                watcher.PortBindingChassisDeletedEvent(self),
+                watcher.SubnetRouterAttachedEvent(self),
+                watcher.SubnetRouterDetachedEvent(self),
+                watcher.TenantPortCreatedEvent(self),
+                watcher.TenantPortDeletedEvent(self),
+                watcher.ChassisCreateEvent(self),
+                watcher.ChassisPrivateCreateEvent(self),
+                watcher.LocalnetCreateDeleteEvent(self)}
 
     @lockutils.synchronized('evpn')
     def frr_sync(self):
@@ -127,8 +113,9 @@ class OVNEVPNDriver(driver_api.AgentDriverBase):
         bridge_mappings = self.ovs_idl.get_ovn_bridge_mappings()
         # 2) Get macs for bridge mappings
         for bridge_index, bridge_mapping in enumerate(bridge_mappings, 1):
-            network = bridge_mapping.split(":")[0]
-            bridge = bridge_mapping.split(":")[1]
+            network, bridge = helpers.parse_bridge_mapping(bridge_mapping)
+            if not network:
+                continue
             self.ovn_bridge_mappings[network] = bridge
 
             linux_net.ensure_arp_ndp_enabled_for_bridge(bridge, bridge_index)
@@ -764,16 +751,17 @@ class OVNEVPNDriver(driver_api.AgentDriverBase):
                 if 'gateway' in route_info['route'].keys():  # subnet route
                     possible_matchings = [
                         r for r in vrf_routes
-                        if (r['dst'] == route_info['route']['dst'] and
+                        if (r.get('dst') == route_info['route']['dst'] and
                             r['dst_len'] == route_info['route']['dst_len'] and
-                            r['gateway'] == route_info['route']['gateway'] and
+                            r.get('gateway') == (
+                                route_info['route']['gateway']) and
                             r['table'] == route_info['route']['table'])]
                 else:  # cr-lrp
                     possible_matchings = [
                         r for r in vrf_routes
-                        if (r['dst'] == route_info['route']['dst'] and
+                        if (r.get('dst') == route_info['route']['dst'] and
                             r['dst_len'] == route_info['route']['dst_len'] and
-                            r['oif'] == oif and
+                            r.get('oif') == oif and
                             r['table'] == route_info['route']['table'])]
                 for r in possible_matchings:
                     vrf_routes.remove(r)
