@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import netaddr
+
 from oslo_config import cfg
 from oslo_log import log as logging
 
@@ -26,7 +28,9 @@ from ovsdbapp.schema.ovn_northbound import impl_idl as nb_impl_idl
 from ovsdbapp.schema.ovn_southbound import impl_idl as sb_impl_idl
 
 from ovn_bgp_agent import constants
+from ovn_bgp_agent.drivers.openstack.utils import driver_utils
 from ovn_bgp_agent import exceptions
+from ovn_bgp_agent.utils import helpers
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
@@ -51,7 +55,8 @@ class OvnDbNotifyHandler(event.RowEventHandler):
 class OvnNbIdl(OvnIdl):
     SCHEMA = 'OVN_Northbound'
 
-    def __init__(self, connection_string, events=None, tables=None):
+    def __init__(self, connection_string, events=None, tables=None,
+                 leader_only=False):
         if connection_string.startswith("ssl"):
             self._check_and_set_ssl_files(self.SCHEMA)
         helper = self._get_ovsdb_helper(connection_string)
@@ -61,15 +66,15 @@ class OvnNbIdl(OvnIdl):
         for table in tables:
             helper.register_table(table)
         super(OvnNbIdl, self).__init__(
-            None, connection_string, helper, leader_only=False)
+            None, connection_string, helper, leader_only=leader_only)
 
     def _get_ovsdb_helper(self, connection_string):
         return idlutils.get_schema_helper(connection_string, self.SCHEMA)
 
     def _check_and_set_ssl_files(self, schema_name):
-        priv_key_file = CONF.ovn_nb_private_key
-        cert_file = CONF.ovn_nb_certificate
-        ca_cert_file = CONF.ovn_nb_ca_cert
+        priv_key_file = CONF.ovn.ovn_nb_private_key
+        cert_file = CONF.ovn.ovn_nb_certificate
+        ca_cert_file = CONF.ovn.ovn_nb_ca_cert
 
         if priv_key_file:
             Stream.ssl_set_private_key_file(priv_key_file)
@@ -114,9 +119,9 @@ class OvnSbIdl(OvnIdl):
         return idlutils.get_schema_helper(connection_string, self.SCHEMA)
 
     def _check_and_set_ssl_files(self, schema_name):
-        priv_key_file = CONF.ovn_sb_private_key
-        cert_file = CONF.ovn_sb_certificate
-        ca_cert_file = CONF.ovn_sb_ca_cert
+        priv_key_file = CONF.ovn.ovn_sb_private_key
+        cert_file = CONF.ovn.ovn_sb_certificate
+        ca_cert_file = CONF.ovn.ovn_sb_ca_cert
 
         if priv_key_file:
             Stream.ssl_set_private_key_file(priv_key_file)
@@ -176,11 +181,214 @@ class LSGetLocalnetPortsCommand(command.ReadOnlyCommand):
             raise RuntimeError(msg) from e
 
 
+# FIXME(ltomasbo): This can be removed once ovsdbapp version is >=2.1.0
+class _LrpNetworksCommand(command.BaseCommand):
+    table = 'Logical_Router_Port'
+
+    def __init__(self, api, port, networks, exists):
+        super().__init__(api)
+        self.port = port
+        self.exists = exists
+        if isinstance(networks, (str, bytes)):
+            networks = [networks]
+        self.networks = [str(netaddr.IPNetwork(network))
+                         for network in networks]
+
+
+# FIXME(ltomasbo): This can be removed once ovsdbapp version is >=2.1.0
+class LrpAddNetworksCommand(_LrpNetworksCommand):
+    def run_idl(self, txn):
+        lrp = self.api.lookup(self.table, self.port)
+        for network in self.networks:
+            if network in lrp.networks and not self.exists:
+                msg = "Network '%s' already exist in networks of port %s" % (
+                    network, lrp.uuid)
+                raise RuntimeError(msg)
+            lrp.addvalue('networks', network)
+
+
+# FIXME(ltomasbo): This can be removed once ovsdbapp supports it
+class LrRouteAddCommand(command.BaseCommand):
+    def __init__(self, api, router, prefix, nexthop, port=None,
+                 policy='dst-ip', ecmp=False, may_exist=False):
+        prefix = str(netaddr.IPNetwork(prefix))
+        if nexthop != constants.ROUTE_DISCARD:
+            nexthop = str(netaddr.IPAddress(nexthop))
+        super().__init__(api)
+        self.router = router
+        self.prefix = prefix
+        self.nexthop = nexthop
+        self.port = port
+        self.policy = policy
+        self.ecmp = ecmp
+        self.may_exist = may_exist
+
+    def run_idl(self, txn):
+        lr = self.api.lookup('Logical_Router', self.router)
+        for route in lr.static_routes:
+            if self.prefix == route.ip_prefix:
+                if self.ecmp and self.nexthop != route.nexthop:
+                    continue
+                if not self.may_exist:
+                    msg = "Route %s already exists on router %s" % (
+                        self.prefix, self.router)
+                    raise RuntimeError(msg)
+                route.nexthop = self.nexthop
+                route.policy = self.policy
+                if self.port:
+                    route.output_port = self.port
+                self.result = rowview.RowView(route)
+                return
+        route = txn.insert(self.api.tables['Logical_Router_Static_Route'])
+        route.ip_prefix = self.prefix
+        route.nexthop = self.nexthop
+        route.policy = self.policy
+        if self.port:
+            route.output_port = self.port
+        lr.addvalue('static_routes', route)
+        self.result = route.uuid
+
+
+# FIXME(ltomasbo): This can be removed once ovsdbapp supports it
+class LrRouteDelCommand(command.BaseCommand):
+    def __init__(self, api, router,
+                 prefix=None, nexthop=None, if_exists=False):
+        if prefix is not None:
+            prefix = str(netaddr.IPNetwork(prefix))
+        super().__init__(api)
+        self.router = router
+        self.prefix = prefix
+        self.nexthop = nexthop
+        self.if_exists = if_exists
+
+    def run_idl(self, txn):
+        lr = self.api.lookup('Logical_Router', self.router)
+        if not self.prefix:
+            lr.static_routes = []
+            return
+        for route in lr.static_routes:
+            if self.prefix == route.ip_prefix:
+                if self.nexthop and route.nexthop != self.nexthop:
+                    continue
+                lr.delvalue('static_routes', route)
+                return
+        if not self.if_exists:
+            msg = "Route for %s in router %s does not exist" % (
+                self.prefix, self.router)
+            raise RuntimeError(msg)
+
+
+class StaticMACBindingFindCommand(command.DbFindCommand):
+    table = 'Static_MAC_Binding'
+
+    def __init__(self, api, port, ip):
+        super().__init__(
+            api,
+            self.table,
+            ('logical_port', '=', port),
+            ('ip', '=', ip),
+            row=True,
+        )
+
+
+# FIXME(ltomasbo): This can be removed once ovsdbapp supports it
+class StaticMACBindingAddCommand(command.AddCommand):
+    table_name = 'Static_MAC_Binding'
+
+    def __init__(self, api, port, ip, mac, override_dynamic_mac=False,
+                 may_exist=False, **columns):
+        super().__init__(api)
+        self.port = port
+        self.ip = ip
+        self.mac = mac
+        self.override_dynamic_mac = override_dynamic_mac
+        self.may_exist = may_exist
+        self.columns = columns
+
+    def run_idl(self, txn):
+        cmd = StaticMACBindingFindCommand(self.api, self.port, self.ip)
+        cmd.run_idl(txn)
+        static_mac_binding_result = cmd.result
+        if static_mac_binding_result:
+            if len(static_mac_binding_result) > 1:
+                # With the current database schema, this cannot happen, but
+                # better safe than sorry.
+                raise RuntimeError(
+                    "Unexpected duplicates in database for port %s "
+                    "and ip %s" % (self.port, self.ip))
+            binding = static_mac_binding_result[0]
+            if self.may_exist:
+                # When no changes are made to a record, the parent
+                # `post_commit` method will not be called.
+                #
+                # Ensure consistent return to caller of `Command.execute()`
+                # even when no changes have been applied.
+                self.result = rowview.RowView(binding)
+                return
+            else:
+                raise RuntimeError(
+                    "Static MAC Binding entry for port %s and ip %s exists" % (
+                        self.port, self.ip))
+        binding = txn.insert(self.api.tables[self.table_name])
+        binding.logical_port = self.port
+        binding.ip = self.ip
+        binding.mac = self.mac
+        binding.override_dynamic_mac = self.override_dynamic_mac
+        self.set_columns(binding, **self.columns)
+        # Setting the result to something other than a :class:`rowview.RowView`
+        # or :class:`ovs.db.idl.Row` typed value will make the parent
+        # `post_commit` method retrieve the newly insterted row from IDL and
+        # return that to the caller.
+        self.result = binding.uuid
+
+
+# FIXME(ltomasbo): This can be removed once ovsdbapp supports it
+class StaticMACBindingDelCommand(command.BaseCommand):
+    table_name = 'Static_MAC_Binding'
+
+    def __init__(self, api, port, ip, if_exists=False, **columns):
+        super().__init__(api)
+        self.port = port
+        self.ip = ip
+        self.if_exists = if_exists
+        self.columns = columns
+
+    def run_idl(self, txn):
+        cmd = StaticMACBindingFindCommand(self.api, self.port, self.ip)
+        cmd.run_idl(txn)
+        static_mac_binding_result = cmd.result
+        if static_mac_binding_result:
+            if len(static_mac_binding_result) > 1:
+                # With the current database schema, this cannot happen, but
+                # better safe than sorry.
+                raise RuntimeError(
+                    "Unexpected duplicates in database for port %s "
+                    "and ip %s" % (self.port, self.ip))
+            binding = static_mac_binding_result[0]
+            binding.delete()
+            return
+        if self.if_exists:
+            return
+        else:
+            raise RuntimeError(
+                "Static MAC Binding entry for port %s and ip %s does not "
+                "exist" % (self.port, self.ip))
+
+
 class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def __init__(self, connection):
         super(OvsdbNbOvnIdl, self).__init__(connection)
         self.idl._session.reconnect.set_probe_interval(60000)
 
+    def get_network_vlan_tags(self):
+        tags = []
+        cmd = self.db_find_rows('Logical_Switch_Port', ('type', '=',
+                                constants.OVN_LOCALNET_VIF_PORT_TYPE))
+        for row in cmd.execute(check_error=True):
+            if row.tag:
+                tags.append(row.tag[0])
+        return tags
+
     def get_network_vlan_tag_by_network_name(self, network_name):
         tags = []
         cmd = self.db_find_rows('Logical_Switch_Port', ('type', '=',
@@ -203,20 +411,105 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         nat_info = cmd.execute(check_error=True)
         return nat_info[0] if nat_info else []
 
-    def get_active_ports_on_chassis(self, chassis):
+    def get_active_lsp_on_chassis(self, chassis):
         ports = []
         cmd = self.db_find_rows('Logical_Switch_Port', ('up', '=', True))
         for row in cmd.execute(check_error=True):
-            if (row.options and
-                    row.options.get('requested-chassis') == chassis):
+            port_chassis, _ = driver_utils.get_port_chassis(row, chassis)
+            if port_chassis == chassis:
+                ports.append(row)
+
+        return ports
+
+    def get_active_cr_lrp_on_chassis(self, chassis):
+        ports = []
+        rows = self.db_list_rows('Logical_Router_Port').execute(
+            check_error=True)
+
+        for row in rows:
+            if not hasattr(row, 'status'):
+                LOG.warning("OVN version does not include support for status "
+                            "information. Therefore router ports and tenant "
+                            "IPs cannot be exposed.")
+                break
+            if row.status.get(constants.OVN_STATUS_CHASSIS) == chassis:
                 ports.append(row)
         return ports
 
+    def get_active_local_lrps(self, local_gateway_ports):
+        ports = []
+        cmd = self.db_find_rows(
+            'Logical_Switch_Port', ('up', '=', True),
+            ('type', '=', constants.OVN_ROUTER_PORT_TYPE),
+            ('external_ids', '=', {constants.OVN_DEVICE_OWNER_EXT_ID_KEY:
+                                   constants.OVN_ROUTER_INTERFACE}))
+        for row in cmd.execute(check_error=True):
+            if (row.external_ids.get(constants.OVN_DEVICE_ID_EXT_ID_KEY)
+                    in local_gateway_ports):
+                ports.append(row)
+        return ports
+
+    def get_active_lsp(self, network):
+        ports = []
+        # port type ""
+        cmd = self.db_find_rows(
+            'Logical_Switch_Port', ('up', '=', True),
+            ('type', '=', constants.OVN_VM_VIF_PORT_TYPE),
+            ('external_ids', '=', {constants.OVN_LS_NAME_EXT_ID_KEY: network}))
+        ports.extend(cmd.execute(check_error=True))
+        # port type "virtual"
+        cmd = self.db_find_rows(
+            'Logical_Switch_Port', ('up', '=', True),
+            ('type', '=', constants.OVN_VIRTUAL_VIF_PORT_TYPE),
+            ('external_ids', '=', {constants.OVN_LS_NAME_EXT_ID_KEY: network}))
+        ports.extend(cmd.execute(check_error=True))
+        return ports
+
+    def get_active_local_lbs(self, local_gateway_ports):
+        lbs = []
+        cmd = self.db_find_rows('Load_Balancer', ('vips', '!=', {}))
+
+        for row in cmd.execute(check_error=True):
+            for ext_id_key in constants.OVN_LB_EXT_ID_ROUTER_KEY:
+                try:
+                    router_name = row.external_ids[ext_id_key].replace(
+                        'neutron-', '', 1)
+                    if router_name in local_gateway_ports:
+                        lbs.append(row)
+                        break
+                except KeyError:
+                    continue
+        return lbs
+
     # FIXME(ltomasbo): This can be removed once ovsdbapp version is >=2.3.0
     def ls_get_localnet_ports(self, logical_switch, if_exists=True):
         return LSGetLocalnetPortsCommand(self, logical_switch,
                                          if_exists=if_exists)
 
+    # FIXME(ltomasbo): This can be removed once ovsdbapp version is >=2.1.0
+    def lrp_add_networks(self, port, networks, may_exist=False):
+        return LrpAddNetworksCommand(self, port, networks, may_exist)
+
+    def lr_route_add(self, router, prefix, nexthop, port=None,
+                     policy='dst-ip', ecmp=False, may_exist=False):
+        return LrRouteAddCommand(self, router, prefix, nexthop, port,
+                                 policy, ecmp, may_exist)
+
+    def lr_route_del(self, router, prefix=None, nexthop=None, if_exists=False):
+        return LrRouteDelCommand(self, router, prefix, nexthop, if_exists)
+
+    def static_mac_binding_add(self, port, ip, mac, override_dynamic_mac=False,
+                               may_exist=False):
+        return StaticMACBindingAddCommand(self, port, ip, mac,
+                                          override_dynamic_mac, may_exist)
+
+    def static_mac_binding_del(self, port, ip, if_exists=False):
+        return StaticMACBindingDelCommand(self, port, ip, if_exists)
+
+    def get_router(self, router):
+        router_name = 'neutron-' + router
+        return self.lr_get(router_name).execute(check_error=True)
+
 
 class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
     def __init__(self, connection):
@@ -288,6 +581,10 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
             pass
         return False
 
+    def is_port_without_chassis(self, port_name):
+        port_info = self.get_port_by_name(port_name)
+        return (port_info and not port_info.chassis)
+
     def is_port_deleted(self, port_name):
         return False if self.get_port_by_name(port_name) else True
 
@@ -340,6 +637,15 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
                 return row.options.get('network_name'), row.tag
         return None, None
 
+    def get_network_vlan_tags(self):
+        tags = []
+        cmd = self.db_find_rows('Port_Binding', ('type', '=',
+                                constants.OVN_LOCALNET_VIF_PORT_TYPE))
+        for row in cmd.execute(check_error=True):
+            if row.tag:
+                tags.append(row.tag[0])
+        return tags
+
     def get_network_vlan_tag_by_network_name(self, network_name):
         tags = []
         cmd = self.db_find_rows('Port_Binding', ('type', '=',
@@ -459,7 +765,7 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
             # This is depending on the external-id information added by
             # neutron, regarding the neutron:cidrs
             ip_info = row.external_ids.get(
-                constants.OVN_CIDRS_EXT_ID_KEY)
+                constants.OVN_CIDRS_EXT_ID_KEY, "")
             if not ip_info:
                 continue
             port_name = row.external_ids.get(
@@ -471,20 +777,24 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
             lb = self.get_ovn_lb(lb_name)
             if not lb:
                 continue
-            if hasattr(lb, 'datapath_group'):
-                lb_dp = lb.datapath_group[0].datapaths
-            else:
+            lb_dp, lr_dp = helpers.get_lb_datapath_groups(lb)
+            if not lb_dp:
                 lb_dp = lb.datapaths
 
-            # assume all the members are connected through the same router
-            # so only one datapath needs to be checked
-            router_lrps = self.get_lrps_for_datapath(lb_dp[0])
-            for lrp in router_lrps:
-                router_lrp_dp = self.get_port_datapath(lrp)
-                if router_lrp_dp == router_dp:
-                    lb_ip = ip_info.split(" ")[0].split("/")[0]
-                    lbs[lb.name] = lb_ip
-                    break
+            if not lr_dp:
+                # assume all the members are connected through the same router
+                # so only one datapath needs to be checked
+                router_lrps = self.get_lrps_for_datapath(lb_dp[0])
+                for lrp in router_lrps:
+                    router_lrp_dp = self.get_port_datapath(lrp)
+                    if router_lrp_dp == router_dp:
+                        lb_ip = ip_info.split(" ")[0].split("/")[0]
+                        lbs[lb.name] = lb_ip
+                        break
+            elif lr_dp == router_dp:
+                lb_ip = ip_info.split(" ")[0].split("/")[0]
+                lbs[lb.name] = lb_ip
+
         return lbs
 
     def get_ovn_vip_port(self, name):
