@@ -12,11 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from oslo_config import cfg
 from oslo_log import log as logging
 from ovs.db import idl
 from ovsdbapp.backend.ovs_idl import connection
 from ovsdbapp.backend.ovs_idl import idlutils
 from ovsdbapp.schema.open_vswitch import impl_idl as idl_ovs
+import socket
 import tenacity
 
 from ovn_bgp_agent import constants
@@ -24,6 +26,7 @@ from ovn_bgp_agent import exceptions as agent_exc
 import ovn_bgp_agent.privileged.ovs_vsctl
 from ovn_bgp_agent.utils import linux_net
 
+CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
@@ -52,12 +55,17 @@ def get_device_port_at_ovs(device):
         'ovs-vsctl', ['get', 'Interface', device, 'ofport'])[0].rstrip()
 
 
-def get_ovs_patch_ports_info(bridge):
-    in_ports = []
+def get_ovs_ports_info(bridge):
     ovs_ports = ovn_bgp_agent.privileged.ovs_vsctl.ovs_cmd(
         'ovs-vsctl', ['list-ports', bridge])[0].rstrip()
-    for ovs_port in ovs_ports.split("\n"):
-        if ovs_port.startswith('patch-provnet-'):
+    return ovs_ports.split("\n")
+
+
+def get_ovs_patch_ports_info(bridge, prefix='patch-provnet-'):
+    in_ports = []
+    ovs_ports = get_ovs_ports_info(bridge)
+    for ovs_port in ovs_ports:
+        if ovs_port.startswith(prefix):
             ovs_ofport = get_device_port_at_ovs(ovs_port)
             in_ports.append(ovs_ofport)
     return in_ports
@@ -133,6 +141,11 @@ def remove_extra_ovs_flows(ovs_flows, bridge, cookie):
                 'ovs-ofctl', ['del-flows', bridge, del_flow])
 
 
+def ensure_flow(bridge, flow):
+    ovn_bgp_agent.privileged.ovs_vsctl.ovs_cmd(
+        'ovs-ofctl', ['add-flow', bridge, flow])
+
+
 def ensure_evpn_ovs_flow(bridge, cookie, mac, output_port, port_dst, net,
                          strip_vlan=False):
     ovs_port = _find_ovs_port(bridge)
@@ -269,33 +282,41 @@ class OvsIdl(object):
     def get_own_chassis_id(self):
         """Return the external_ids:system-id value of the Open_vSwitch table.
 
-        As long as ovn-controller is running on this node, the key is
-        guaranteed to exist and will include the chassis name.
         """
         return self._get_from_ext_ids('system-id')
 
     def get_own_chassis_name(self):
         """Return the external_ids:hostname value of the Open_vSwitch table.
 
-        As long as ovn-controller is running on this node, the key is
-        guaranteed to exist and will include the chassis name.
+        If the value is not configured, it will fetch the hostname from the
+        current machine.
         """
-        return self._get_from_ext_ids('hostname')
+        try:
+            return self._get_from_ext_ids('hostname')
+        except KeyError:
+            return socket.gethostname()
 
-    def get_ovn_remote(self):
+    def get_ovn_remote(self, nb=False):
         """Return the external_ids:ovn-remote value of the Open_vSwitch table.
 
         """
-        return self._get_from_ext_ids('ovn-remote')
+        if nb:
+            return (CONF.ovn.ovn_nb_connection if CONF.ovn.ovn_nb_connection
+                    else self._get_from_ext_ids('ovn-nb-remote'))
+        return (CONF.ovn.ovn_sb_connection if CONF.ovn.ovn_sb_connection
+                else self._get_from_ext_ids('ovn-remote'))
 
-    def get_ovn_bridge_mappings(self):
+    def get_ovn_bridge_mappings(self, bridge=None):
         """Return a list of bridge mappings
 
         Return a list of bridge mappings based on the
         external_ids:ovn-bridge-mappings value of the Open_vSwitch table.
         """
+        key = 'ovn-bridge-mappings'
+        if bridge:
+            key = key + '-' + str(bridge)
         try:
             return [i.strip() for i in
-                    self._get_from_ext_ids('ovn-bridge-mappings').split(',')]
+                    self._get_from_ext_ids(key).split(',')]
         except KeyError:
             return []
