@@ -16,10 +16,9 @@ from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_log import log as logging
 
-from ovsdbapp.backend.ovs_idl import event as row_event
-
 from ovn_bgp_agent import constants
 from ovn_bgp_agent.drivers.openstack.watchers import base_watcher
+from ovn_bgp_agent.utils import helpers
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
@@ -37,12 +36,21 @@ class PortBindingChassisCreatedEvent(base_watcher.PortBindingChassisEvent):
             # single and dual-stack format
             if not self._check_ip_associated(row.mac[0]):
                 return False
-            return (row.chassis[0].name == self.agent.chassis and
-                    (not old.chassis or row.chassis != old.chassis))
+            if not bool(row.up[0]):
+                return False
+
+            if row.chassis[0].name != self.agent.chassis:
+                return False
+            if hasattr(old, 'chassis'):
+                if not old.chassis or row.chassis != old.chassis:
+                    return True
+            if hasattr(old, 'up'):
+                if not bool(old.up[0]):
+                    return True
         except (IndexError, AttributeError):
             return False
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         if row.type not in constants.OVN_VIF_PORT_TYPES:
             return
         with _SYNC_STATE_LOCK.read_lock():
@@ -61,15 +69,25 @@ class PortBindingChassisDeletedEvent(base_watcher.PortBindingChassisEvent):
             # single and dual-stack format
             if not self._check_ip_associated(row.mac[0]):
                 return False
-            if event == self.ROW_UPDATE:
-                return (old.chassis[0].name == self.agent.chassis and
-                        (not row.chassis or row.chassis != old.chassis))
-            else:
+            if event == self.ROW_DELETE:
                 return row.chassis[0].name == self.agent.chassis
+
+            if hasattr(old, 'chassis'):
+                if (old.chassis[0].name == self.agent.chassis and
+                        (not row.chassis or row.chassis != old.chassis)):
+                    return True
+            if hasattr(old, 'up'):
+                # this requires to have unchanged chassis and being the local
+                # one. If there was a chassis change, then it was already
+                # processed before
+                if (row.chassis[0].name == self.agent.chassis and
+                        bool(old.up[0]) and not bool(row.up[0])):
+                    return True
         except (IndexError, AttributeError):
             return False
+        return False
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         if row.type not in constants.OVN_VIF_PORT_TYPES:
             return
         with _SYNC_STATE_LOCK.read_lock():
@@ -91,7 +109,7 @@ class FIPSetEvent(base_watcher.PortBindingChassisEvent):
         except (AttributeError):
             return False
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         if row.type != constants.OVN_PATCH_VIF_PORT_TYPE:
             return
         with _SYNC_STATE_LOCK.read_lock():
@@ -129,7 +147,7 @@ class FIPUnsetEvent(base_watcher.PortBindingChassisEvent):
         except (AttributeError):
             return False
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         if row.type != constants.OVN_PATCH_VIF_PORT_TYPE:
             return
         with _SYNC_STATE_LOCK.read_lock():
@@ -171,7 +189,7 @@ class SubnetRouterAttachedEvent(base_watcher.PortBindingChassisEvent):
         except (IndexError, AttributeError):
             return False
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         if row.type != constants.OVN_PATCH_VIF_PORT_TYPE:
             return
         with _SYNC_STATE_LOCK.read_lock():
@@ -206,7 +224,7 @@ class SubnetRouterUpdateEvent(base_watcher.PortBindingChassisEvent):
         except (IndexError, AttributeError):
             return False
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         if row.type != constants.OVN_PATCH_VIF_PORT_TYPE:
             return
         with _SYNC_STATE_LOCK.read_lock():
@@ -230,7 +248,7 @@ class SubnetRouterDetachedEvent(base_watcher.PortBindingChassisEvent):
         except (IndexError, AttributeError):
             return False
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         if row.type != constants.OVN_PATCH_VIF_PORT_TYPE:
             return
         with _SYNC_STATE_LOCK.read_lock():
@@ -249,7 +267,8 @@ class TenantPortCreatedEvent(base_watcher.PortBindingChassisEvent):
             # Handling the case for unknown MACs when configdrive is used
             # instead of dhcp
             if row.mac == ['unknown']:
-                n_cidrs = row.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY)
+                n_cidrs = row.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY,
+                                               "").split()
                 if not n_cidrs:
                     return False
             # single and dual-stack format
@@ -260,7 +279,7 @@ class TenantPortCreatedEvent(base_watcher.PortBindingChassisEvent):
         except (IndexError, AttributeError):
             return False
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         if row.type not in (constants.OVN_VM_VIF_PORT_TYPE,
                             constants.OVN_VIRTUAL_VIF_PORT_TYPE):
             return
@@ -268,7 +287,8 @@ class TenantPortCreatedEvent(base_watcher.PortBindingChassisEvent):
             if row.mac == ['unknown']:
                 # Handling the case for unknown MACs when configdrive is used
                 # instead of dhcp
-                n_cidrs = row.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY)
+                n_cidrs = row.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY,
+                                               "")
                 ips = [ip.split("/")[0] for ip in n_cidrs.split(" ")]
             else:
                 ips = row.mac[0].split(' ')[1:]
@@ -286,7 +306,8 @@ class TenantPortDeletedEvent(base_watcher.PortBindingChassisEvent):
             if row.mac == ['unknown']:
                 # Handling the case for unknown MACs when configdrive is used
                 # instead of dhcp
-                n_cidrs = row.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY)
+                n_cidrs = row.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY,
+                                               "").split()
                 if not n_cidrs:
                     return False
             # single and dual-stack format
@@ -300,7 +321,7 @@ class TenantPortDeletedEvent(base_watcher.PortBindingChassisEvent):
         except (IndexError, AttributeError):
             return False
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         if row.type not in (constants.OVN_VM_VIF_PORT_TYPE,
                             constants.OVN_VIRTUAL_VIF_PORT_TYPE):
             return
@@ -312,7 +333,8 @@ class TenantPortDeletedEvent(base_watcher.PortBindingChassisEvent):
             if row.mac == ['unknown']:
                 # Handling the case for unknown MACs when configdrive is used
                 # instead of dhcp
-                n_cidrs = row.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY)
+                n_cidrs = row.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY,
+                                               "")
                 ips = [ip.split("/")[0] for ip in n_cidrs.split(" ")]
             else:
                 ips = row.mac[0].split(' ')[1:]
@@ -337,14 +359,15 @@ class OVNLBVIPPortEvent(base_watcher.PortBindingChassisEvent):
         except (IndexError, AttributeError):
             return False
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         if row.type != constants.OVN_VM_VIF_PORT_TYPE:
             return
 
         with _SYNC_STATE_LOCK.read_lock():
             # This is depending on the external-id information added by
             # neutron, regarding the neutron:cidrs
-            ext_n_cidr = row.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY)
+            ext_n_cidr = row.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY,
+                                              "")
             if not ext_n_cidr:
                 return
 
@@ -355,7 +378,7 @@ class OVNLBVIPPortEvent(base_watcher.PortBindingChassisEvent):
                 self.agent.expose_ovn_lb(ovn_lb_ip, row)
 
 
-class OVNLBMemberCreateEvent(base_watcher.OVNLBMemberEvent):
+class OVNLBMemberCreateEvent(base_watcher.OVNLBEvent):
     def __init__(self, bgp_agent):
         events = (self.ROW_CREATE,)
         super(OVNLBMemberCreateEvent, self).__init__(
@@ -365,7 +388,7 @@ class OVNLBMemberCreateEvent(base_watcher.OVNLBMemberEvent):
         # Only process event if the local node has a cr-lrp ports associated
         return bool(self.agent.ovn_local_cr_lrps)
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         # Only process event if the local node has a cr-lrp port whose provider
         # datapath is included into the loadbalancer. This means the
         # loadbalancer has the VIP on a provider network
@@ -377,12 +400,8 @@ class OVNLBMemberCreateEvent(base_watcher.OVNLBMemberEvent):
             row_dp = row.datapaths
         except AttributeError:
             row_dp = []
-        if hasattr(row, 'datapath_group'):
-            if row.datapath_group:
-                dp_datapaths = row.datapath_group[0].datapaths
-                if dp_datapaths:
-                    row_dp = dp_datapaths
 
+        row_dp, router_dps = helpers.get_lb_datapath_groups(row)
         if not row_dp:
             # No need to continue. There is no need to expose it as there is
             # no datapaths (aka members).
@@ -390,15 +409,14 @@ class OVNLBMemberCreateEvent(base_watcher.OVNLBMemberEvent):
         vip_port = self.agent.sb_idl.get_ovn_vip_port(row.name)
         if not vip_port:
             return
-        vip_ip = vip_port.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY)
+        vip_ip = vip_port.external_ids.get(constants.OVN_CIDRS_EXT_ID_KEY, "")
         if not vip_ip:
             return
         vip_ip = vip_ip.strip().split(" ")[0].split("/")[0]
         associated_cr_lrp_port = None
 
-        router_dps = []
-        if not (CONF.expose_tenant_networks or
-                CONF.expose_ipv6_gua_tenant_networks):
+        if not router_dps and not (CONF.expose_tenant_networks or
+                                   CONF.expose_ipv6_gua_tenant_networks):
             # assume all the members are connected through the same router
             # so only one member needs to be checked
             member_dp = row_dp[0]
@@ -428,7 +446,7 @@ class OVNLBMemberCreateEvent(base_watcher.OVNLBMemberEvent):
                 vip_ip, row.name, associated_cr_lrp_port)
 
 
-class OVNLBMemberDeleteEvent(base_watcher.OVNLBMemberEvent):
+class OVNLBMemberDeleteEvent(base_watcher.OVNLBEvent):
     def __init__(self, bgp_agent):
         events = (self.ROW_DELETE,)
         super(OVNLBMemberDeleteEvent, self).__init__(
@@ -438,7 +456,7 @@ class OVNLBMemberDeleteEvent(base_watcher.OVNLBMemberEvent):
         # Only process event if the local node has the lb exported
         return bool(self.agent.provider_ovn_lbs.get(row.name))
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         associated_cr_lrp_port = self.agent.provider_ovn_lbs[row.name].get(
             'gateway_port')
         if not associated_cr_lrp_port:
@@ -459,12 +477,12 @@ class LocalnetCreateDeleteEvent(base_watcher.PortBindingChassisEvent):
     def match_fn(self, event, row, old):
         return row.type == constants.OVN_LOCALNET_VIF_PORT_TYPE
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         with _SYNC_STATE_LOCK.read_lock():
             self.agent.sync()
 
 
-class ChassisCreateEventBase(row_event.RowEvent):
+class ChassisCreateEventBase(base_watcher.Event):
     table = None
 
     def __init__(self, bgp_agent):
@@ -475,7 +493,7 @@ class ChassisCreateEventBase(row_event.RowEvent):
             events, self.table, (('name', '=', self.agent.chassis),))
         self.event_name = self.__class__.__name__
 
-    def run(self, event, row, old):
+    def _run(self, event, row, old):
         if self.first_time:
             self.first_time = False
         else:
