@@ -17,13 +17,13 @@ from unittest import mock
 
 from oslo_config import cfg
 
-from ovn_bgp_agent import config
 from ovn_bgp_agent import constants
 from ovn_bgp_agent.drivers.openstack import ovn_stretched_l2_bgp_driver
 from ovn_bgp_agent.drivers.openstack.utils import driver_utils
 from ovn_bgp_agent.drivers.openstack.utils import frr
 from ovn_bgp_agent.drivers.openstack.utils import ovn
 from ovn_bgp_agent.drivers.openstack.utils import ovs
+from ovn_bgp_agent import exceptions as agent_exc
 from ovn_bgp_agent.tests import base as test_base
 from ovn_bgp_agent.tests.unit import fakes
 from ovn_bgp_agent.utils import linux_net
@@ -55,7 +55,6 @@ class TestHashedRoute(test_base.TestCase):
 class TestOVNBGPStretchedL2Driver(test_base.TestCase):
     def setUp(self):
         super(TestOVNBGPStretchedL2Driver, self).setUp()
-        config.register_opts()
         CONF.set_override(
             "address_scopes",
             "11111111-1111-1111-1111-11111111,22222222-2222-2222-2222-22222222",  # NOQA E501
@@ -136,19 +135,26 @@ class TestOVNBGPStretchedL2Driver(test_base.TestCase):
         self.mock_ndb = mock.patch.object(linux_net.pyroute2, "NDB").start()
         self.fake_ndb = self.mock_ndb().__enter__()
 
+    @mock.patch.object(linux_net, "ensure_vrf")
     @mock.patch.object(linux_net, "ensure_ovn_device")
     @mock.patch.object(linux_net, "delete_routes_from_table")
     @mock.patch.object(frr, "vrf_leak")
-    def test_start(self, mock_vrf, mock_delete_routes, mock_ensure_ovn_device):
+    def test_start(self, mock_vrf, mock_delete_routes, mock_ensure_ovn_device,
+                   *args):
         CONF.set_override("clear_vrf_routes_on_startup", True)
 
+        mock_redistribute = mock.patch.object(
+            frr, "set_default_redistribute"
+        ).start()
+
         self.bgp_driver.start()
 
+        mock_redistribute.assert_called_with(['kernel'])
         mock_vrf.assert_called_once_with(
             CONF.bgp_vrf,
             CONF.bgp_AS,
             CONF.bgp_router_id,
-            template=frr.LEAK_VRF_KERNEL_TEMPLATE,
+            template=frr.LEAK_VRF_TEMPLATE,
         )
         # Assert connections were started
         self.mock_ovs_idl().start.assert_called_once_with(
@@ -159,20 +165,26 @@ class TestOVNBGPStretchedL2Driver(test_base.TestCase):
         mock_ensure_ovn_device.assert_called_once_with(
             CONF.bgp_nic, CONF.bgp_vrf)
 
+    @mock.patch.object(linux_net, "ensure_vrf")
     @mock.patch.object(linux_net, "ensure_ovn_device")
     @mock.patch.object(linux_net, "delete_routes_from_table")
     @mock.patch.object(frr, "vrf_leak")
     def test_start_clear_routes(
-        self, mock_vrf, mock_delete_routes, mock_ensure_ovn_device):
+        self, mock_vrf, mock_delete_routes, mock_ensure_ovn_device, *args):
         CONF.set_override("clear_vrf_routes_on_startup", False)
 
+        mock_redistribute = mock.patch.object(
+            frr, "set_default_redistribute"
+        ).start()
+
         self.bgp_driver.start()
 
+        mock_redistribute.assert_called_with(['kernel'])
         mock_vrf.assert_called_once_with(
             CONF.bgp_vrf,
             CONF.bgp_AS,
             CONF.bgp_router_id,
-            template=frr.LEAK_VRF_KERNEL_TEMPLATE,
+            template=frr.LEAK_VRF_TEMPLATE,
         )
         # Assert connections were started
         self.mock_ovs_idl().start.assert_called_once_with(
@@ -357,6 +369,23 @@ class TestOVNBGPStretchedL2Driver(test_base.TestCase):
 
         mock__ensure_network_exposed.assert_not_called()
 
+    def test_expose_subnet_no_datapath(self):
+        mock__ensure_network_exposed = mock.patch.object(
+            self.bgp_driver, "_ensure_network_exposed"
+        ).start()
+        row = mock.Mock()
+        row.datapath = "fake-dp"
+        self.sb_idl.is_router_gateway_on_any_chassis.side_effect = (
+            agent_exc.DatapathNotFound(datapath=row.datapath))
+
+        self.bgp_driver.expose_subnet(None, row)
+
+        self.sb_idl.is_router_gateway_on_any_chassis.assert_called_once_with(
+            row.datapath
+        )
+
+        mock__ensure_network_exposed.assert_not_called()
+
     def test_update_subnet(self):
         mock__update_network = mock.patch.object(
             self.bgp_driver, "_update_network"
@@ -381,6 +410,31 @@ class TestOVNBGPStretchedL2Driver(test_base.TestCase):
             row, self.cr_lrp0.logical_port, ["3.3.3.3/24"], ["1.1.1.1/24"]
         )
 
+    def test_update_subnet_no_datapath(self):
+        mock__update_network = mock.patch.object(
+            self.bgp_driver, "_update_network"
+        ).start()
+        self.sb_idl.is_router_gateway_on_any_chassis.return_value = (
+            self.cr_lrp0
+        )
+        old = mock.Mock()
+        old.mac = ["ff:ff:ff:ff:ff:01 1.1.1.1/24 2.2.2.2/24"]
+
+        row = mock.Mock()
+        row.datapath = "fake-dp"
+        row.mac = ["ff:ff:ff:ff:ff:01 2.2.2.2/24 3.3.3.3/24"]
+
+        self.sb_idl.is_router_gateway_on_any_chassis.side_effect = (
+            agent_exc.DatapathNotFound(datapath=row.datapath))
+
+        self.bgp_driver.update_subnet(old, row)
+
+        self.sb_idl.is_router_gateway_on_any_chassis.assert_called_once_with(
+            row.datapath
+        )
+
+        mock__update_network.assert_not_called()
+
     @mock.patch.object(linux_net, "get_exposed_routes_on_network")
     @mock.patch.object(linux_net, "del_ip_route")
     @mock.patch.object(linux_net, "add_ip_route")
@@ -566,7 +620,7 @@ class TestOVNBGPStretchedL2Driver(test_base.TestCase):
         }
         self.bgp_driver.ovn_local_cr_lrps = {
             self.cr_lrp0.logical_port: {
-                "lrp_ports": set([row.logical_port, "another_lrp_port"])
+                "lrp_ports": {row.logical_port, "another_lrp_port"}
             }
         }
 
@@ -585,7 +639,7 @@ class TestOVNBGPStretchedL2Driver(test_base.TestCase):
             self.bgp_driver.ovn_local_cr_lrps,
             {
                 self.cr_lrp0.logical_port: {
-                    "lrp_ports": set(["another_lrp_port"])
+                    "lrp_ports": {"another_lrp_port"}
                 }
             }
         )
@@ -835,7 +889,7 @@ class TestOVNBGPStretchedL2Driver(test_base.TestCase):
             for ip in ["10.0.0.10/26", "fd51:f4b3:872:eda::10/64"]
         ]
         gateway["address_scopes"] = self.addr_scope
-        gateway["lrp_ports"] = set([""])
+        gateway["lrp_ports"] = {""}
         self.bgp_driver.ovn_local_cr_lrps = {"gateway_port": gateway}
         port_info = {
             "cr_lrp": self.cr_lrp0.logical_port,
@@ -976,23 +1030,23 @@ class TestOVNBGPStretchedL2Driver(test_base.TestCase):
         self.bgp_driver.ovn_local_cr_lrps = {
             self.cr_lrp0.logical_port: gateway
         }
-        gateway["lrp_ports"] = set([
+        gateway["lrp_ports"] = {
             "lrp-lrp_port0",
             "lrp-lrp_port1",
             "lrp-lrp_port2",
-        ])
+        }
 
         lrp_port0 = {
             "cr_lrp": self.cr_lrp0.logical_port,
-            "subnets": set([
+            "subnets": {
                 "fdcc:8cf2:d40c:1::/64",
-                "192.168.0.0/24"])
+                "192.168.0.0/24"}
         }
         lrp_port1 = {
             "cr_lrp": self.cr_lrp0.logical_port,
-            "subnets": set([
+            "subnets": {
                 "fdcc:8cf2:d40c:2::/64",
-                "192.168.1.0/24"])
+                "192.168.1.0/24"}
         }
 
         self.bgp_driver.propagated_lrp_ports = {
@@ -1031,11 +1085,11 @@ class TestOVNBGPStretchedL2Driver(test_base.TestCase):
                 constants.IP_VERSION_6: '',
             }
         }
-        gateway["lrp_ports"] = set([
+        gateway["lrp_ports"] = {
             "lrp-lrp_port0",
             "lrp-lrp_port1",
             "lrp-lrp_port2",
-        ])
+        }
         self.bgp_driver.propagated_lrp_ports = {
             "lrp-lrp_port0": {},
             "lrp-lrp_port1": {},
