@@ -20,7 +20,6 @@ from ovs.stream import Stream
 from ovsdbapp.backend.ovs_idl import connection
 from ovsdbapp.backend.ovs_idl import idlutils
 
-from ovn_bgp_agent import config
 from ovn_bgp_agent import constants
 from ovn_bgp_agent.drivers.openstack.utils import ovn as ovn_utils
 from ovn_bgp_agent import exceptions
@@ -38,8 +37,22 @@ class TestOvsdbNbOvnIdl(test_base.TestCase):
 
         # Monkey-patch parent class methods
         self.nb_idl.db_find_rows = mock.Mock()
+        self.nb_idl.db_list_rows = mock.Mock()
         self.nb_idl.lookup = mock.Mock()
 
+    def test_get_network_vlan_tags(self):
+        tag = [123]
+        lsp = fakes.create_object({'name': 'port-0',
+                                   'tag': tag})
+        self.nb_idl.db_find_rows.return_value.execute.return_value = [
+            lsp]
+        ret = self.nb_idl.get_network_vlan_tags()
+
+        self.assertEqual(tag, ret)
+        self.nb_idl.db_find_rows.assert_called_once_with(
+            'Logical_Switch_Port',
+            ('type', '=', constants.OVN_LOCALNET_VIF_PORT_TYPE))
+
     def test_get_network_vlan_tag_by_network_name(self):
         network_name = 'net0'
         tag = [123]
@@ -87,21 +100,140 @@ class TestOvsdbNbOvnIdl(test_base.TestCase):
             'NAT',
             ('logical_port', '=', logical_port))
 
-    def test_get_active_ports_on_chassis(self):
+    def test_get_active_lsp_on_chassis_options(self):
         chassis = 'local_chassis'
         row1 = fakes.create_object({
-            'options': {'requested-chassis': chassis}})
+            'options': {'requested-chassis': chassis},
+            'external_ids': {}})
         row2 = fakes.create_object({
-            'options': {'requested-chassis': 'other_chassis'}})
+            'options': {'requested-chassis': 'other_chassis'},
+            'external_ids': {}})
         self.nb_idl.db_find_rows.return_value.execute.return_value = [
             row1, row2]
-        ret = self.nb_idl.get_active_ports_on_chassis(chassis)
+        ret = self.nb_idl.get_active_lsp_on_chassis(chassis)
 
         self.assertEqual([row1], ret)
         self.nb_idl.db_find_rows.assert_called_once_with(
             'Logical_Switch_Port',
             ('up', '=', True))
 
+    def test_get_active_lsp_on_chassis_external_ids(self):
+        chassis = 'local_chassis'
+        row1 = fakes.create_object({
+            'options': {},
+            'external_ids': {'neutron:host_id': chassis}})
+        row2 = fakes.create_object({
+            'options': {},
+            'external_ids': {'neutron:host_id': 'other_chassis'}})
+        self.nb_idl.db_find_rows.return_value.execute.return_value = [
+            row1, row2]
+        ret = self.nb_idl.get_active_lsp_on_chassis(chassis)
+
+        self.assertEqual([row1], ret)
+        self.nb_idl.db_find_rows.assert_called_once_with(
+            'Logical_Switch_Port',
+            ('up', '=', True))
+
+    def test_get_active_cr_lrp_on_chassis(self):
+        chassis = 'local_chassis'
+        row1 = fakes.create_object({
+            'status': {'hosting-chassis': 'local_chassis'}})
+        row2 = fakes.create_object({
+            'status': {'hosting-chassis': 'other_chassis'}})
+        row3 = fakes.create_object({})
+        self.nb_idl.db_list_rows.return_value.execute.return_value = [
+            row1, row2, row3]
+
+        ret = self.nb_idl.get_active_cr_lrp_on_chassis(chassis)
+
+        self.assertEqual([row1], ret)
+        self.nb_idl.db_list_rows.assert_called_once_with(
+            'Logical_Router_Port')
+
+    def test_get_active_local_lrps(self):
+        local_gateway_ports = ['router1']
+        row1 = fakes.create_object({
+            'external_ids': {
+                constants.OVN_DEVICE_OWNER_EXT_ID_KEY:
+                    constants.OVN_ROUTER_INTERFACE,
+                constants.OVN_DEVICE_ID_EXT_ID_KEY: 'router1'
+            }})
+        row2 = fakes.create_object({
+            'external_ids': {
+                constants.OVN_DEVICE_OWNER_EXT_ID_KEY:
+                    constants.OVN_ROUTER_INTERFACE,
+                constants.OVN_DEVICE_ID_EXT_ID_KEY: 'other_router'
+            }})
+
+        self.nb_idl.db_find_rows.return_value.execute.return_value = [
+            row1, row2]
+        ret = self.nb_idl.get_active_local_lrps(local_gateway_ports)
+
+        self.assertEqual([row1], ret)
+        self.nb_idl.db_find_rows.assert_called_once_with(
+            'Logical_Switch_Port',
+            ('up', '=', True), ('type', '=', constants.OVN_ROUTER_PORT_TYPE),
+            ('external_ids', '=', {constants.OVN_DEVICE_OWNER_EXT_ID_KEY:
+                                   constants.OVN_ROUTER_INTERFACE}))
+
+    def test_get_active_lsp(self):
+        row1 = fakes.create_object({
+            'type': constants.OVN_VM_VIF_PORT_TYPE,
+            'external_ids': {constants.OVN_LS_NAME_EXT_ID_KEY: 'net1'}})
+        row2 = fakes.create_object({
+            'type': constants.OVN_VIRTUAL_VIF_PORT_TYPE,
+            'external_ids': {constants.OVN_LS_NAME_EXT_ID_KEY: 'net1'}})
+        self.nb_idl.db_find_rows.return_value.execute.side_effect = [
+            [row1], [row2]]
+
+        ret = self.nb_idl.get_active_lsp('net1')
+
+        self.assertEqual([row1, row2], ret)
+        expected_calls = [
+            mock.call('Logical_Switch_Port', ('up', '=', True),
+                      ('type', '=', constants.OVN_VM_VIF_PORT_TYPE),
+                      ('external_ids', '=',
+                       {constants.OVN_LS_NAME_EXT_ID_KEY: 'net1'})),
+            mock.call().execute(check_error=True),
+            mock.call('Logical_Switch_Port', ('up', '=', True),
+                      ('type', '=', constants.OVN_VIRTUAL_VIF_PORT_TYPE),
+                      ('external_ids', '=',
+                       {constants.OVN_LS_NAME_EXT_ID_KEY: 'net1'})),
+            mock.call().execute(check_error=True)]
+
+        self.nb_idl.db_find_rows.assert_has_calls(expected_calls)
+
+    def test_get_active_local_lbs(self):
+        local_gateway_ports = ['router1']
+        lb1 = fakes.create_object({
+            'vips': {'vip': 'member1,member2'},
+            'external_ids': {
+                constants.OVN_LB_LR_REF_EXT_ID_KEY: "neutron-router1"}})
+        lb2 = fakes.create_object({
+            'vips': {'vip': 'member1,member2'},
+            'external_ids': {
+                constants.OVN_LB_LR_REF_EXT_ID_KEY: "neutron-router2"}})
+        self.nb_idl.db_find_rows.return_value.execute.return_value = [lb1, lb2]
+
+        ret = self.nb_idl.get_active_local_lbs(local_gateway_ports)
+
+        self.assertEqual([lb1], ret)
+        self.nb_idl.db_find_rows.assert_called_once_with(
+            'Load_Balancer', ('vips', '!=', {}))
+
+        self.nb_idl.db_find_rows.reset_mock()
+
+        lb3 = fakes.create_object({
+            'vips': {'fip': 'member1'},
+            'external_ids': {
+                constants.OVN_LR_NAME_EXT_ID_KEY: "neutron-router1"}})
+        self.nb_idl.db_find_rows.return_value.execute.return_value = [
+            lb1, lb2, lb3]
+        ret = self.nb_idl.get_active_local_lbs(local_gateway_ports)
+        self.assertEqual([lb1, lb3], ret)
+        self.nb_idl.db_find_rows.assert_called_once_with(
+            'Load_Balancer', ('vips', '!=', {}))
+
 
 class TestOvsdbSbOvnIdl(test_base.TestCase):
 
@@ -232,6 +364,22 @@ class TestOvsdbSbOvnIdl(test_base.TestCase):
             self.assertFalse(self.sb_idl.is_port_on_chassis(
                 'fake-port', 'fake-chassis'))
 
+    def test_is_port_without_chassis(self):
+        chassis_name = 'fake-chassis'
+        with mock.patch.object(self.sb_idl, 'get_port_by_name') as mock_p:
+            ch = fakes.create_object({'name': chassis_name})
+            mock_p.return_value = fakes.create_object(
+                {'type': constants.OVN_VM_VIF_PORT_TYPE,
+                 'chassis': [ch]})
+            self.assertFalse(self.sb_idl.is_port_without_chassis('fake-port'))
+
+    def test_is_port_without_chassis_no_chassis(self):
+        with mock.patch.object(self.sb_idl, 'get_port_by_name') as mock_p:
+            mock_p.return_value = fakes.create_object(
+                {'type': constants.OVN_VM_VIF_PORT_TYPE,
+                 'chassis': []})
+            self.assertTrue(self.sb_idl.is_port_without_chassis('fake-port'))
+
     def _test_is_port_deleted(self, port_exist=True):
         ret_value = mock.Mock() if port_exist else []
         with mock.patch.object(self.sb_idl, 'get_port_by_name') as mock_p:
@@ -323,6 +471,14 @@ class TestOvsdbSbOvnIdl(test_base.TestCase):
     def test_get_network_name_and_tag_not_in_bridge_mappings(self):
         self._test_get_network_name_and_tag(network_in_bridge_map=False)
 
+    def test_get_netweork_vlan_tags(self):
+        tag = [1001]
+        row = fakes.create_object({'tag': tag})
+        self.sb_idl.db_find_rows.return_value.execute.return_value = [row, ]
+
+        ret = self.sb_idl.get_network_vlan_tags()
+        self.assertEqual(tag, ret)
+
     def _test_get_network_vlan_tag_by_network_name(self, match=True):
         network = 'public' if match else 'spongebob'
         tag = [1001]
@@ -640,7 +796,6 @@ class TestOvnNbIdl(test_base.TestCase):
 
     def setUp(self):
         super(TestOvnNbIdl, self).setUp()
-        config.register_opts()
         mock.patch.object(idlutils, 'get_schema_helper').start()
         mock.patch.object(ovn_utils.OvnIdl, '__init__').start()
         self.nb_idl = ovn_utils.OvnNbIdl('tcp:127.0.0.1:6640')
@@ -650,9 +805,9 @@ class TestOvnNbIdl(test_base.TestCase):
     @mock.patch.object(Stream, 'ssl_set_private_key_file')
     def test__check_and_set_ssl_files(
             self, mock_ssl_priv_key, mock_ssl_cert, mock_ssl_ca_cert):
-        CONF.set_override('ovn_nb_private_key', 'fake-priv-key')
-        CONF.set_override('ovn_nb_certificate', 'fake-cert')
-        CONF.set_override('ovn_nb_ca_cert', 'fake-ca-cert')
+        CONF.set_override('ovn_nb_private_key', 'fake-priv-key', group='ovn')
+        CONF.set_override('ovn_nb_certificate', 'fake-cert', group='ovn')
+        CONF.set_override('ovn_nb_ca_cert', 'fake-ca-cert', group='ovn')
 
         self.nb_idl._check_and_set_ssl_files('fake-schema')
 
@@ -677,7 +832,6 @@ class TestOvnSbIdl(test_base.TestCase):
 
     def setUp(self):
         super(TestOvnSbIdl, self).setUp()
-        config.register_opts()
         mock.patch.object(idlutils, 'get_schema_helper').start()
         mock.patch.object(ovn_utils.OvnIdl, '__init__').start()
         self.sb_idl = ovn_utils.OvnSbIdl('tcp:127.0.0.1:6640')
@@ -687,9 +841,9 @@ class TestOvnSbIdl(test_base.TestCase):
     @mock.patch.object(Stream, 'ssl_set_private_key_file')
     def test__check_and_set_ssl_files(
             self, mock_ssl_priv_key, mock_ssl_cert, mock_ssl_ca_cert):
-        CONF.set_override('ovn_sb_private_key', 'fake-priv-key')
-        CONF.set_override('ovn_sb_certificate', 'fake-cert')
-        CONF.set_override('ovn_sb_ca_cert', 'fake-ca-cert')
+        CONF.set_override('ovn_sb_private_key', 'fake-priv-key', group='ovn')
+        CONF.set_override('ovn_sb_certificate', 'fake-cert', group='ovn')
+        CONF.set_override('ovn_sb_ca_cert', 'fake-ca-cert', group='ovn')
 
         self.sb_idl._check_and_set_ssl_files('fake-schema')
 
