@@ -17,7 +17,6 @@ from unittest import mock
 
 from oslo_config import cfg
 
-from ovn_bgp_agent import config
 from ovn_bgp_agent import constants
 from ovn_bgp_agent.drivers.openstack.watchers import bgp_watcher
 from ovn_bgp_agent.tests import base as test_base
@@ -37,7 +36,8 @@ class TestPortBindingChassisCreatedEvent(test_base.TestCase):
     def test_match_fn(self):
         ch = utils.create_row(name=self.chassis)
         row = utils.create_row(chassis=[ch],
-                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'])
+                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'],
+                               up=[True])
         old = utils.create_row(chassis=[])
         self.assertTrue(self.event.match_fn(mock.Mock(), row, old))
 
@@ -51,14 +51,16 @@ class TestPortBindingChassisCreatedEvent(test_base.TestCase):
     def test_match_fn_old_chassis_set(self):
         ch = utils.create_row(name=self.chassis)
         row = utils.create_row(chassis=[ch],
-                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'])
+                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'],
+                               up=[True])
         old = utils.create_row(chassis=[ch])
         self.assertFalse(self.event.match_fn(mock.Mock(), row, old))
 
     def test_match_fn_no_old_chassis(self):
         ch = utils.create_row(name=self.chassis)
         row = utils.create_row(chassis=[ch],
-                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'])
+                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'],
+                               up=[True])
         old = utils.create_row(chassis=[])
         self.assertTrue(self.event.match_fn(mock.Mock(), row, old))
 
@@ -66,10 +68,27 @@ class TestPortBindingChassisCreatedEvent(test_base.TestCase):
         ch = utils.create_row(name=self.chassis)
         ch_old = utils.create_row(name='old-chassis')
         row = utils.create_row(chassis=[ch],
-                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'])
+                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'],
+                               up=[True])
         old = utils.create_row(chassis=[ch_old])
         self.assertTrue(self.event.match_fn(mock.Mock(), row, old))
 
+    def test_match_fn_no_up(self):
+        ch = utils.create_row(name=self.chassis)
+        row = utils.create_row(chassis=[ch],
+                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'],
+                               up=[False])
+        self.assertFalse(self.event.match_fn(mock.Mock(), row, mock.Mock))
+
+    def test_match_fn_no_old_up(self):
+        ch = utils.create_row(name=self.chassis)
+        row = utils.create_row(chassis=[ch],
+                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'],
+                               up=[True])
+        old = utils.create_row(chassis=[ch],
+                               up=[False])
+        self.assertTrue(self.event.match_fn(mock.Mock(), row, old))
+
     def test_match_fn_index_error(self):
         row = utils.create_row(chassis=[],
                                mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'])
@@ -106,11 +125,12 @@ class TestPortBindingChassisDeletedEvent(test_base.TestCase):
         self.event = bgp_watcher.PortBindingChassisDeletedEvent(self.agent)
 
     def test_match_fn(self):
+        event = self.event.ROW_DELETE
         ch = utils.create_row(name=self.chassis)
         row = utils.create_row(chassis=[ch],
                                mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'])
         old = utils.create_row(chassis=[])
-        self.assertTrue(self.event.match_fn(mock.Mock(), row, old))
+        self.assertTrue(self.event.match_fn(event, row, old))
 
     def test_match_fn_not_single_or_dual_stack(self):
         ch = utils.create_row(name=self.chassis)
@@ -127,12 +147,33 @@ class TestPortBindingChassisDeletedEvent(test_base.TestCase):
         old = utils.create_row(chassis=[ch])
         self.assertTrue(self.event.match_fn(event, row, old))
 
-    def test_match_fn_update_old_chassis_set(self):
+    def test_match_fn_update_old_chassis_set_up_false(self):
         event = self.event.ROW_UPDATE
         ch = utils.create_row(name=self.chassis)
         row = utils.create_row(chassis=[ch],
-                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'])
-        old = utils.create_row(chassis=[ch])
+                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'],
+                               up=[True])
+        old = utils.create_row(chassis=[ch],
+                               up=[False])
+        self.assertFalse(self.event.match_fn(event, row, old))
+
+    def test_match_fn_update_old_chassis_set_up_true(self):
+        event = self.event.ROW_UPDATE
+        ch = utils.create_row(name=self.chassis)
+        row = utils.create_row(chassis=[ch],
+                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'],
+                               up=[False])
+        old = utils.create_row(chassis=[ch],
+                               up=[True])
+        self.assertTrue(self.event.match_fn(event, row, old))
+
+    def test_match_fn_update_different_chassis_set_up_false(self):
+        event = self.event.ROW_UPDATE
+        ch = utils.create_row(name='other-chassis')
+        row = utils.create_row(chassis=[ch],
+                               mac=['aa:bb:cc:dd:ee:ff 10.10.1.16'],
+                               up=[False])
+        old = utils.create_row(up=[True])
         self.assertFalse(self.event.match_fn(event, row, old))
 
     def test_match_fn_update_no_chassis(self):
@@ -840,7 +881,6 @@ class TestOVNLBMemberCreateEvent(test_base.TestCase):
 
     def setUp(self):
         super(TestOVNLBMemberCreateEvent, self).setUp()
-        config.register_opts()
         self.chassis = '935f91fa-b8f8-47b9-8b1b-3a7a90ef7c26'
         self.agent = mock.Mock(chassis=self.chassis)
         self.agent.ovn_local_cr_lrps = {
@@ -882,7 +922,7 @@ class TestOVNLBMemberCreateEvent(test_base.TestCase):
         dpg1 = utils.create_row(_uuid='fake_dp_group',
                                 datapaths=['s_dp1'])
         row = utils.create_row(name='ovn-lb1',
-                               datapath_group=[dpg1],
+                               ls_datapath_group=[dpg1],
                                vips={'172.24.100.66:80': '10.0.0.5:8080'})
         self.agent.ovn_local_cr_lrps = {
             'cr-lrp1': {'provider_datapath': 'dp1',
@@ -905,7 +945,7 @@ class TestOVNLBMemberCreateEvent(test_base.TestCase):
         dpg1 = utils.create_row(_uuid='fake_dp_group',
                                 datapaths=['s_dp1'])
         row = utils.create_row(name='ovn-lb1',
-                               datapath_group=[dpg1],
+                               lr_datapath_group=[dpg1],
                                vips={'172.24.100.66:80': '10.0.0.5:8080'})
         self.agent.sb_idl.get_ovn_vip_port.return_value = []
         self.event.run(self.event.ROW_CREATE, row, mock.Mock())
@@ -933,7 +973,7 @@ class TestOVNLBMemberCreateEvent(test_base.TestCase):
         dpg1 = utils.create_row(_uuid='fake_dp_group',
                                 datapaths=['s_dp2'])
         row = utils.create_row(name='ovn-lb1',
-                               datapath_group=[dpg1],
+                               ls_datapath_group=[dpg1],
                                vips={'172.24.100.66:80': '10.0.0.5:8080'})
         vip_port = utils.create_row(
             datapath='dp1',
@@ -950,7 +990,7 @@ class TestOVNLBMemberCreateEvent(test_base.TestCase):
         dpg1 = utils.create_row(_uuid='fake_dp_group',
                                 datapaths=['s_dp1'])
         row = utils.create_row(name='ovn-lb1',
-                               datapath_group=[dpg1],
+                               lr_datapath_group=[dpg1],
                                vips={'172.24.100.66:80': '10.0.0.5:8080'})
         vip_port = utils.create_row(
             datapath='dp1',
